<?php

namespace Drupal\openweathermap\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Locale\CountryManagerInterface;
use Drupal\openweathermap\CityManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * The controller for the weather forecast routes.
 */
class DisplayWeatherController extends ControllerBase {

  /**
   * Country manager service.
   *
   * @var \Drupal\Core\Locale\CountryManagerInterface
   */
  protected $countryManager;

  /**
   * City manager service.
   *
   * @var \Drupal\openweathermap\CityManagerInterface
   */
  protected $cityManager;

  /**
   * Language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * API endpoint.
   *
   * @var string
   */
  protected $apiEndpoint;

  /**
   * API key.
   *
   * @var string
   */
  protected $apiKey;

  /**
   * Units of measurement, see https://openweathermap.org/current#data .
   *
   * @var string
   */
  protected $units;

  /**
   * Current language.
   *
   * @var string
   */
  protected $language;

  /**
   * Country code.
   *
   * @var string
   */
  protected $countryCode;

  /**
   * Service URL for requesting weather forecast data.
   *
   * @var string
   */
  protected $url;

  /**
   * Constructs a new DisplayWeather object.
   */
  public function __construct(CountryManagerInterface $country_manager, CityManagerInterface $city_manager, LanguageManagerInterface $language_manager, ConfigFactoryInterface $config_factory) {
    $this->countryManager = $country_manager;
    $this->cityManager = $city_manager;
    $this->languageManager = $language_manager;
    $this->config = $config_factory->get('openweathermap.settings');

    $this->apiEndpoint = $this->config->get('api_endpoint');
    $this->apiKey = $this->config->get('api_key');
    $this->units = $this->config->get('units_of_measurement');
    $this->language = $this->languageManager->getCurrentLanguage()->getId();
  }

  /**
   * {@inheritdoc}
   */
  public function displayWeatherDefaultLocation() {
    $this->countryCode = $this->config->get('country_code');

    $city_id = $this->config->get('city_id');

    $args = "?id={$city_id}" . "&units={$this->units}" . "&appid={$this->apiKey}" . "&lang={$this->language}";
    $this->url = $this->apiEndpoint . $args;

    return $this->response();
  }

  /**
   * {@inheritdoc}
   */
  public function displayWeatherByCityName($city) {
    $this->countryCode = NULL;

    $args = "?q={$city}" . "&units={$this->units}" . "&appid={$this->apiKey}" . "&lang={$this->language}";
    $this->url = $this->apiEndpoint . $args;

    return $this->response();
  }

  /**
   * To test this method, "Brest" city can be used.
   *
   * Brest city is presents in Belarus, Germany and France. Possible urls are:
   * /weather/Brest/by
   * /weather/Brest/de
   * /weather/Bresr/fr
   * .
   */
  public function displayWeatherByCityNameAndCountryCode($city, $country_code) {
    $this->countryCode = strtoupper($country_code);

    $args = "?q={$city},{$country_code}" . "&units={$this->units}" . "&appid={$this->apiKey}" . "&lang={$this->language}";
    $this->url = $this->apiEndpoint . $args;

    return $this->response();
  }

  /**
   * {@inheritdoc}
   */
  protected function response() {

    if ($this->apiKey == '') {

      $this->messenger()
        ->addWarning("Sorry, but the weather service hasn't been configured yet. Please contact your system administrator.");

      return $this->redirect('<front>');
    }

    // Get a country name to display instead of a country code.
    if ($this->countryCode) {
      $country_name = $this->countryManager->getList()[$this->countryCode]->__toString();
    }
    else {
      $country_name = '';
    }

    $build['#theme'] = 'openweathermap_weather_page';
    $build['#attached']['library'] = 'openweathermap/openweathermap.display_weather';

    $build['#attached']['drupalSettings'] = [
      'url' => $this->url,
      'country_name' => $country_name,
      'units_of_measurement' => $this->units,
      'update_periodically' => $this->config->get('update_periodically'),
      // Translate minutes to milliseconds.
      'update_interval' => $this->config->get('update_interval') * 60000,
      'weather_characteristics' => $this->config->get('weather_characteristics'),
    ];

    return $build;
  }

}
