<?php

namespace Drupal\openweathermap\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The settings form for the 'Openweathermap' module.
 */
class OpenweathermapSettingsForm extends ConfigFormBase {

  /**
   * The configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * All countries from the 'city.list.json' file.
   *
   * @var array
   */
  protected $countries;

  /**
   * City manager service.
   *
   * @var \Drupal\openweathermap\CityManagerInterface
   */
  protected $cityManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->cityManager = $container->get('city_manager');
    $instance->config = $instance->config('openweathermap.settings');
    $instance->countries = $instance->cityManager->getCountries();
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'openweathermap.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openweathermap_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['#attached']['library'] = 'openweathermap/openweathermap.select2_for_settings_form';

    if (isset($form_state->getUserInput()['country'])) {

      $county_code = $form_state->getUserInput()['country'];
      $city_id = NULL;
    }
    else {
      $county_code = $this->config->get('country_code');
      $city_id = $this->config->get('city_id');
    }

    $form['country'] = [
      '#type' => 'select',
      '#title' => $this->t('Country'),
      '#options' => $this->countries,
      '#size' => 7,
      '#default_value' => $county_code,
      '#required' => TRUE,
      '#ajax' => [
        'event' => 'change',
        'callback' => '::changeCoutryAjaxCallback',
        'wrapper' => 'city-select-wrapper',
      ],
    ];

    $cities = $this->cityManager->getCitiesByCountryCode($county_code);

    $form['city'] = [
      '#type' => 'select',
      '#title' => $this->t('City'),
      '#options' => $cities,
      '#size' => 7,
      '#required' => TRUE,
      '#default_value' => $city_id,
      '#prefix' => '<div id="city-select-wrapper">',
      '#suffix' => '</div>',
      // To prevent the error message: "An illegal choice has been detected...".
      '#validated' => TRUE,
    ];

    $form['units_container'] = [
      '#type' => 'fieldset',
    ];

    $form['units_container']['units_of_measurement'] = [
      '#type' => 'select',
      '#title' => $this->t('Units of measurement'),
      '#options' => [
        'standard' => $this->t('standard'),
        'metric' => $this->t('metric'),
        'imperial' => $this->t('imperial'),
      ],
      '#default_value' => $this->config->get('units_of_measurement'),
      '#required' => TRUE,
    ];

    $form['api_container'] = [
      '#type' => 'fieldset',
    ];

    $form['api_container']['api_endpoint'] = [
      '#type' => 'url',
      '#title' => $this->t('API Endpoint'),
      '#size' => 60,
      '#default_value' => $this->config->get('api_endpoint'),
      '#required' => TRUE,
    ];

    $form['api_container']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#default_value' => $this->config->get('api_key'),
      '#size' => 60,
      '#required' => TRUE,
    ];

    $form['update_container'] = [
      '#type' => 'fieldset',
    ];

    $form['update_container']['update_periodically'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Update periodically'),
      '#description' => $this->t('If is set, then a weather forecast data will be updated at the specified time interval.'),
      '#default_value' => $this->config->get('update_periodically'),
    ];

    $form['update_container']['update_interval'] = [
      '#type' => 'number',
      '#title' => $this->t('Update interval in minutes'),
      '#min' => 1,
      '#default_value' => $this->config->get('update_interval'),
      '#states' => [
        'enabled' => [
          ':input[name="update_periodically"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function changeCoutryAjaxCallback($form, $form_state) {
    return $form['city'];
  }

  /**
   * Implement the validator because we were forced to set '#validated' => TRUE.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    if (empty($form_state->getValue('city'))) {
      $form_state->setErrorByName('city', $this->t('The "City" field is required.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config->set('country_code', $form_state->getValue('country'));
    $this->config->set('city_id', $form_state->getValue('city'));
    $this->config->set('units_of_measurement', $form_state->getValue('units_of_measurement'));
    $this->config->set('api_endpoint', $form_state->getValue('api_endpoint'));
    $this->config->set('api_key', $form_state->getValue('api_key'));
    $this->config->set('update_periodically', $form_state->getValue('update_periodically'));
    $this->config->set('update_interval', $form_state->getValue('update_interval'));

    $this->config->save();
  }

}
